==> jugg-agile-spring-boot-webmvc-2.0-agile-spring-boot-webmvc-20250418.063054-25-sources.jar: com/jugg/agile/spring/boot/webmvc/servlet/JaServletFilter.java <==
package com.jugg.agile.spring.boot.webmvc.servlet;

import com.jugg.agile.framework.core.config.JaProperty;
import com.jugg.agile.framework.core.config.JaPropertyListener;
import com.jugg.agile.framework.core.context.biz.JaCoreContext;
import com.jugg.agile.framework.core.context.biz.JaCoreContextEntity;
import com.jugg.agile.framework.core.dapper.alarm.JaAlarm;
import com.jugg.agile.framework.core.dapper.alarm.JaAlarmMessage;
import com.jugg.agile.framework.core.dapper.alarm.adapter.qywx.JaQyWxAlarm;
import com.jugg.agile.framework.core.dapper.log.JaLog;
import com.jugg.agile.framework.core.dapper.log.JaMDC;
import com.jugg.agile.framework.core.meta.function.JaFunctionP;
import com.jugg.agile.framework.core.util.JaStringUtil;
import com.jugg.agile.framework.core.util.JaThrowableUtil;
import com.jugg.agile.framework.core.util.io.net.http.meta.JaHttpStatus;
import com.jugg.agile.framework.core.util.io.serialize.json.JaJson;
import com.jugg.agile.framework.meta.dto.JaResultDTO;
import com.jugg.agile.framework.meta.exception.JaException;
import com.jugg.agile.spring.boot.webmvc.context.JaWebContextChain;
import com.jugg.agile.spring.boot.webmvc.util.JaServletUtil;
import lombok.SneakyThrows;
import org.springframework.http.HttpHeaders;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.nio.charset.StandardCharsets;
import java.util.HashSet;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * controller 前置处理, 以防三方包内置controller未拦截到
 *
 * @author chenjian
 * @since 2024年01月05日 19:35:06
 */
public class JaServletFilter implements Filter {

    private static final String logFormatEntry = "url:{},context:{}";

    private static long servletConnNums;
    private static boolean servletConnNumsStop;
    private static boolean checkUnknownUpServer;
    private static final JaQyWxAlarm alarmUnknownUpServer;
    private static final HashSet<String> unknownUpServerIPSet = new HashSet<>();

    private static final AtomicInteger Connections = new AtomicInteger();

    static {
        alarmUnknownUpServer = new JaQyWxAlarm("throwable");
        alarmUnknownUpServer.setUrl("https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key=c8e9019c-fb68-4281-ba47-690be69fb47a");
        JaPropertyListener.addAndRunCommonListener(() -> {
            checkUnknownUpServer = JaProperty.getBoolean("ja.servlet.checkUnknownUpServer", false);
        });
    }


    @Override
    public void init(FilterConfig filterConfig) {
        JaWebContextChain instance = JaWebContextChain.getInstance();
        JaLog.info("JaServletFilter init:{}", instance);
    }

    @Override
    @SneakyThrows
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain chain) {
//        int connNums = Connections.incrementAndGet();
        HttpServletRequest request = (HttpServletRequest) servletRequest;
        HttpServletResponse response = (HttpServletResponse) servletResponse;
        String requestURI = request.getRequestURI();
        try {
            JaWebContextChain.getInstance().inherit(request);

            request.setCharacterEncoding(StandardCharsets.UTF_8.name());


            if (JaStringUtil.isEmpty(response.getHeader(JaMDC.TraceId))) {
                response.setHeader(HttpHeaders.ACCESS_CONTROL_EXPOSE_HEADERS, JaMDC.TraceId);
                response.addHeader(JaMDC.TraceId, JaMDC.get());
            }


            if (JaServletConfig.isPass(requestURI)) {
                chain.doFilter(servletRequest, servletResponse);
                return;
            }


            JaLog.info(logFormatEntry, requestURI, JaJson.toString(JaCoreContext.getInstance().get()));
            checkUpServer(request);


            chain.doFilter(servletRequest, servletResponse);
        } catch (Throwable e) {
            handleException(request, response, e);
        } finally {
//            alarm(connNums);
            JaWebContextChain.getInstance().remove();
        }
    }

    @Override
    public void destroy() {
        JaLog.info("JaServletFilter destroy");
    }

    protected static void handleException(HttpServletRequest request, HttpServletResponse response, Throwable e) {
        JaLog.get().error("http filter catch ex", e);
        e = JaThrowableUtil.getCause(e);
        String code = null;
        String codeMsg = null;
        // TODO 未做国际化
        if (e instanceof JaException) {
            code = ((JaException) e).getCode();
        }
//        if (JaStringUtil.isEmpty(code)) {
//            codeMsg = JaI18nUtil.getMessage("Unknown Error"
//                    , request.getRequestURI()
//                    , JaMDC.get());
//        }
        codeMsg = String.format("Unknown Error:[%s]-[%s]"
                , request.getRequestURI()
                , JaMDC.get());
        JaResultDTO<Object> result = new JaResultDTO<>();
        result.setCode(code);
        result.setCodeMsg(codeMsg);
        response.setStatus(JaHttpStatus.HTTP_VERSION_NOT_SUPPORTED);
        JaServletUtil.printResponse(request, response, result);
    }

    private void alarm(int connNums) {
        if (servletConnNumsStop && connNums > servletConnNums) {
            try {
                String error = String.format("servlet current thread : %s > %s", connNums, servletConnNums);
                JaLog.info(error);
                JaFunctionP<String> alarmHandler = JaAlarm.getAlarmHandler();
                if (null != alarmHandler) {
                    alarmHandler.apply(JaAlarmMessage.getCommonMessage() + error);
                }
            } catch (Throwable e) {
                JaLog.error("servlet connection nums alarm error", e);
            }
        }
        Connections.decrementAndGet();
    }

    /**
     * 检查未知上游服务告警
     */
    private void checkUpServer(HttpServletRequest request) {
        JaCoreContextEntity jaCoreContextEntity = JaCoreContext.getInstance().get();
        String usIp = jaCoreContextEntity.getUsIp();
        if (checkUnknownUpServer
                && JaStringUtil.isEmpty(jaCoreContextEntity.getUsName())
                && JaStringUtil.isNotEmpty(usIp)
                && !JaServletUtil.isFromWeb(request)
                && !unknownUpServerIPSet.contains(usIp)
        ) {
            alarmUnknownUpServer.alarm(JaAlarmMessage.getCommonMessage() + "未知来源IP:" + jaCoreContextEntity.getUsIp());
            unknownUpServerIPSet.add(jaCoreContextEntity.getUsIp());
            if (unknownUpServerIPSet.size() > 10000) {
                alarmUnknownUpServer.alarm(JaAlarmMessage.getCommonMessage() + "未知来源IP数清空缓存");
                unknownUpServerIPSet.clear();
            }
        }
    }
}

==> jugg-agile-spring-boot-webmvc-2.6.13.0-agile-spring-boot-webmvc-20250814.092041-2-sources.jar: com/jugg/agile/spring/boot/webmvc/servlet/JaServletFilter.java <==
package com.jugg.agile.spring.boot.webmvc.servlet;

import com.jugg.agile.framework.core.config.JaProperty;
import com.jugg.agile.framework.core.config.JaPropertyListener;
import com.jugg.agile.framework.core.context.biz.JaCoreContext;
import com.jugg.agile.framework.core.context.biz.JaCoreContextEntity;
import com.jugg.agile.framework.core.dapper.alarm.JaAlarm;
import com.jugg.agile.framework.core.dapper.alarm.JaAlarmMessage;
import com.jugg.agile.framework.core.dapper.alarm.JaAlarmWebhook;
import com.jugg.agile.framework.core.dapper.alarm.adapter.qywx.JaQyWxAlarm;
import com.jugg.agile.framework.core.dapper.log.JaLog;
import com.jugg.agile.framework.core.dapper.log.JaMDC;
import com.jugg.agile.framework.core.meta.function.JaFunctionP;
import com.jugg.agile.framework.core.util.JaStringUtil;
import com.jugg.agile.framework.core.util.JaThrowableUtil;
import com.jugg.agile.framework.core.util.io.serialize.json.JaJson;
import com.jugg.agile.framework.meta.dto.JaResultDTO;
import com.jugg.agile.framework.meta.exception.JaException;
import com.jugg.agile.spring.boot.webmvc.context.JaWebContextChain;
import com.jugg.agile.spring.boot.webmvc.util.JaServletUtil;
import lombok.SneakyThrows;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.nio.charset.StandardCharsets;
import java.util.HashSet;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * controller 前置处理, 以防三方包内置controller未拦截到
 *
 * @author chenjian
 * @since 2024年01月05日 19:35:06
 */
public class JaServletFilter implements Filter {

    private static final String logFormatEntry = "url:{},context:{}";

    private static long servletConnNums;
    private static boolean servletConnNumsStop;
    private static boolean checkUnknownUpServer;
    private static final JaQyWxAlarm alarmUnknownUpServer;
    private static final HashSet<String> unknownUpServerIPSet = new HashSet<>();

    private static final AtomicInteger Connections = new AtomicInteger();

    static {
        alarmUnknownUpServer = new JaQyWxAlarm("checkUpServer");
        alarmUnknownUpServer.setUrl("https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key=c8e9019c-fb68-4281-ba47-690be69fb47a");
        JaPropertyListener.addAndRunCommonListener(() -> {
            checkUnknownUpServer = JaProperty.getBoolean("ja.servlet.checkUnknownUpServer", false);
        });
    }


    @Override
    public void init(FilterConfig filterConfig) {
        JaWebContextChain instance = JaWebContextChain.getInstance();
        JaLog.info("JaServletFilter init:{}", instance);
    }

    @Override
    @SneakyThrows
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain chain) {
//        int connNums = Connections.incrementAndGet();
        HttpServletRequest request = (HttpServletRequest) servletRequest;
        HttpServletResponse response = (HttpServletResponse) servletResponse;
        String requestURI = request.getRequestURI();
        try {
            JaWebContextChain.getInstance().inherit(request);

            request.setCharacterEncoding(StandardCharsets.UTF_8.name());


            if (JaStringUtil.isEmpty(response.getHeader(JaMDC.TraceId))) {
                response.setHeader(HttpHeaders.ACCESS_CONTROL_EXPOSE_HEADERS, JaMDC.TraceId);
                response.addHeader(JaMDC.TraceId, JaMDC.get());
            }


            if (JaServletConfig.isPass(requestURI)) {
                chain.doFilter(servletRequest, servletResponse);
                return;
            }


            JaLog.info(logFormatEntry, requestURI, JaJson.toString(JaCoreContext.getInstance().get()));
            checkUpServer(request);


            chain.doFilter(servletRequest, servletResponse);
        } catch (Throwable e) {
            handleException(request, response, e);
        } finally {
//            alarm(connNums);
            JaWebContextChain.getInstance().remove();
        }
    }

    @Override
    public void destroy() {
        JaAlarmWebhook.ScheduledThreadPoolExecutor.shutdownNow();
        JaLog.info("JaServletFilter destroy");
    }

    protected static void handleException(HttpServletRequest request, HttpServletResponse response, Throwable e) {
        JaLog.get().error("http filter catch ex", e);
        e = JaThrowableUtil.getCause(e);
        String code = null;
        String codeMsg = null;
        // TODO 未做国际化
        if (e instanceof JaException) {
            code = ((JaException) e).getCode();
        }
//        if (JaStringUtil.isEmpty(code)) {
//            codeMsg = JaI18nUtil.getMessage("Unknown Error"
//                    , request.getRequestURI()
//                    , JaMDC.get());
//        }
        codeMsg = String.format("Unknown Error:[%s]-[%s]"
                , request.getRequestURI()
                , JaMDC.get());
        JaResultDTO<Object> result = new JaResultDTO<>();
        result.setCode(code);
        result.setCodeMsg(codeMsg);
        response.setStatus(HttpStatus.HTTP_VERSION_NOT_SUPPORTED.value());
        JaServletUtil.printResponse(request, response, result);
    }

    private void alarm(int connNums) {
        if (servletConnNumsStop && connNums > servletConnNums) {
            try {
                String error = String.format("servlet current thread : %s > %s", connNums, servletConnNums);
                JaLog.info(error);
                JaFunctionP<String> alarmHandler = JaAlarm.getAlarmHandler();
                if (null != alarmHandler) {
                    alarmHandler.apply(JaAlarmMessage.getCommonMessage() + error);
                }
            } catch (Throwable e) {
                JaLog.error("servlet connection nums alarm error", e);
            }
        }
        Connections.decrementAndGet();
    }

    /**
     * 检查未知上游服务告警
     */
    private void checkUpServer(HttpServletRequest request) {
        JaCoreContextEntity jaCoreContextEntity = JaCoreContext.getInstance().get();
        String usIp = jaCoreContextEntity.getUsIp();
        if (checkUnknownUpServer
                && JaStringUtil.isEmpty(jaCoreContextEntity.getUsName())
                && JaStringUtil.isNotEmpty(usIp)
                && !JaServletUtil.isFromWeb(request)
                && !unknownUpServerIPSet.contains(usIp)
        ) {
            alarmUnknownUpServer.alarm(JaAlarmMessage.getCommonMessage() + "未知来源IP:" + jaCoreContextEntity.getUsIp());
            unknownUpServerIPSet.add(jaCoreContextEntity.getUsIp());
            if (unknownUpServerIPSet.size() > 10000) {
                alarmUnknownUpServer.alarm(JaAlarmMessage.getCommonMessage() + "未知来源IP数清空缓存");
                unknownUpServerIPSet.clear();
            }
        }
    }
}
